#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Fichero manejador de ficheros
#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Módulo de interfaces"""

class GestorConfig:
    """
    Gestor de configuracion. Se encarga de controlar si esta cargada la configuracion
    si el formato es el más actual, y mantiene una referencia al propio objeto
    de configuración
    """
    def __init__(self):
        import os
        self.__fichero = os.environ["HOME"]+"/.driza"
        self.configuracion = None

    def load(self, fichero = None):
        """Carga un fichero de configuracion, por defecto ~/.driza"""
        from Driza.excepciones import VersionAnterior
        nuevo = False
        if fichero:
            self.__fichero = fichero
        ifc = InterfazConfig()
        configuracion = None
        try:
            configuracion = ifc.load(self.__fichero)
        except IOError:
            nuevo = True
            configuracion = Configuracion()
            ifc.save(configuracion, self.__fichero)
        except VersionAnterior(configuracion):
            #Aqui van las correcciones para cada revision
            if configuracion["version"] < 2:
                del configuracion["primeravez"]
                configuracion["nundo"] = 5 #TODO: Ver si esto hace falta 
                configuracion["decimales"] = 3
            configuracion["version"] = 2

            self.configuracion = configuracion
        
        self.configuracion = configuracion
        return nuevo #Devuelve si el fichero es nuevo o no
            
    def save(self, fichero = None):
        """Guarda el fichero de configuracion en un fichero"""
        if fichero:
            self.__fichero = fichero
        ifc = InterfazConfig()
        ifc.save(self.configuracion, self.__fichero)


class InterfazConfig:
    """
    Clase encargada de actuar como interfaz de la configuración. Permite 
    leer y guardar la configuración en ficheros.
    """
    def load(self, fichero):
        """
        Carga la configuración del fichero de configuración del programa, 
        devolviendo la configuración por defecto si no la encuentra. 
        El parametro fichero es opcional e indica un fichero alternativo 
        al de configuracion por defecto 
        """
        from ConfigParser import ConfigParser
        configfile = ConfigParser()
        config = Configuracion()
        archivo = open(fichero,"r")
        configfile.readfp(archivo)
        config.creaDesdeTexto(configfile._sections["General"])
        if config["version"] < 2:
            from Driza.excepciones import VersionAnterior
            raise VersionAnterior(configuracion)
        archivo.close()
        return config


    def save(self, config, fichero):
        """
        Guarda la configuracion en el fichero de configuración del programa. 
        El parametro fichero es opcional e indica un fichero alternativo al 
        de configuracion por defecto
        """
        from ConfigParser import ConfigParser
        configfile = ConfigParser()
        config["lfichero"] = ",".join(config["lfichero"])
        configfile.add_section("General")

        for oper, valor in config.items():
            configfile.set("General", oper, valor)

        archivo = open(fichero, "w")
        configfile.write(archivo)
        archivo.close()

        #Volvemos a poner a un tipo ListaFicheros
        from Driza.clasesbase import ListaFicheros
        config["lfichero"] = ListaFicheros(5, config["lfichero"].split(","))

    # Elementos a configurar:

    #tmpdir: Guarda la ruta del directorio de trabajo
    #vsplash: boolean que dice si aparece la ventana splash o no
    #lfichero: Lista de ficheros
    #version es un entero que representa la version del formato de configuración

class Configuracion(dict):
    def __init__(self):
        from clasesbase import ListaFicheros
        dict.__init__(self)
        listaficheros = ListaFicheros(5)
        self.__setitem__("version", 2)
        self.__setitem__("tmpdir", "/tmp")
        self.__setitem__("lfichero", listaficheros)
        self.__setitem__("decimales", 3)
        self.__setitem__("nundo", 5)
        self.__setitem__("vsplash", True)

    def creaDesdeTexto(self,diccionario):
        from clasesbase import ListaFicheros
        self.__setitem__("version", diccionario["version"])
        self.__setitem__("tmpdir", diccionario["tmpdir"])
        self.__setitem__("lfichero", ListaFicheros(5,diccionario["lfichero"].split(',')))
        self.__setitem__("decimales", eval(diccionario["decimales"]))
        self.__setitem__("nundo", eval(diccionario["nundo"]))
        self.__setitem__("vsplash", eval(diccionario['vsplash']))
