#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Definicion de agrupadores de variables"""

class Agrupador:
    """Clase de tipo que es padre de todos los Agrupadores.
    Un agrupador son aquellas caracteristicas de una variable de estudio
    """
    def __init__(self,nombre,tipovariableasociado,ristratipo,diccionarioconversion={},descripcion="",valorpordefecto=None):
        self.nombre=nombre
        self.descripcion=descripcion
        self.diccionarioconversion=diccionarioconversion
        self.tipovariableasociado=tipovariableasociado
        self.valorpordefecto=valorpordefecto
        self.tipo=ristratipo

    def update(self,objeto):
        """Función de copia de los atributos para cualquier Agrupador.
        Sirve para hacer copias entre Agrupadores
        """
        self.nombre=objeto.nombre
        self.descripcion=objeto.descripcion

    def nuevoitem(self,valor):
        """Crea un nuevo item del tipo indicado"""
        return self.tipovariableasociado(self,valor)

class AgrupadorDiscreto(Agrupador):
    """Sirve para saber si la clase es discreta o no"""
    pass

class AgrupadorNumerico(Agrupador):
    """Sirve para saber si la clase es numerica o no"""
    pass

class Etiquetable:
    """Añade las etiquetas a los agrupadores"""
    def __init__(self):
        self.etiquetas={}

class AgrupadorFactor(AgrupadorDiscreto):
    """Clase de tipo que representa a los factores"""
    from variables import Factor
    def __init__(self,nombre,Factor,descripcion=""):
        Agrupador.__init__(self,nombre,"Factor",{"Agrupador":["CDefault"]},descripcion,"")

class AgrupadorEntero(AgrupadorDiscreto,AgrupadorNumerico,Etiquetable):
    """
    La clase entero. Almacena cualquier número entero,
    garantizando las operaciones de transformación a formato interno y a ristra.
    """
    def __init__(self,nombre,valorpordefecto="NA",descripcion=""):
        """ Inicializacion de TVariable entera """
        from variables import Entero
        Agrupador.__init__(self,nombre,Entero,"Entero",{"Agrupador":["CDefault"],"AgrupadorOrdinal":["CIgualdad"],"AgrupadorReal":["CIgualdad"]},descripcion,valorpordefecto)
        Etiquetable.__init__(self)
    
class AgrupadorOrdinal(AgrupadorDiscreto,Etiquetable):
    """ La clase ordinal. Almacena cualquier número entero,
    garantizando las operaciones de transformación a formato interno y a ristra.
    """
    def __init__(self,nombre,valorpordefecto=None,descripcion=""):
        """ Inicializacion de TVariable Ordinal """
        from variables import Ordinal
        Agrupador.__init__(self,nombre,Ordinal,"Ordinal",{"Agrupador":["CDefault"],"AgrupadorEntero":["CIgualdad"],"AgrupadorReal":["CIgualdad"]},descripcion,valorpordefecto)
        Etiquetable.__init__(self)
    
class AgrupadorReal(AgrupadorNumerico):
    """
    La clase Real. Almacena cualquier número real, 
    garantizando las operaciones de transformación a formato interno y a ristra. 
    """
    def __init__(self,nombre,valorpordefecto=None,descripcion="",numerodecimales=3):
        """Inicializacion del objeto Real"""
        from variables import Real
        Agrupador.__init__(self,nombre,Real,"Real",{"Agrupador":["CDefault"],"AgrupadorEntero":["CRedondeoInferior"],"AgrupadorOrdinal":["CRedondeoInferior"]},descripcion,valorpordefecto)
        self.numerodecimales=3

class AgrupadorLogico(AgrupadorDiscreto):
    """ Almacena variables lógicas """
    def __init__(self,nombre,valorpordefecto=None,descripcion=""):
        """Inicializacion del objeto Real"""
        from variables import Logico
        Agrupador.__init__(self,nombre,Logico,"Logico",{"Agrupador":["CDefault"]},descripcion,valorpordefecto)


