#!/usr/bin/python
# -*- coding: utf-8 -*-
#

#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Interfaces de acceso a los datos
"""
import logging
log = logging.getLogger("Driza.datos.interfaces")

class InterfazDatos:
    """
    Interfazdatos pretende ser una interfaz homogenea de acceso y modificacion
    de los datos para grid y otros elementos del programa
    La clase ofrece multiples formas de accesso a los datos internos
    """

    def __init__(self, portero):
        """Inicialización del contenedor principal"""
        self._portero = portero

    #FUNCIONES PUBLICAS

    def __getitem__(self, indice): #Acceso por []
        """
        Devuelve el item segun un indice numerico, un nombre, 
        o un objeto variable
        Si el indice es un indice, se entiende que se quiere 
        acceder a un registro
        Si el indice es un nombre o el objeto variable, se entiende 
        que se quiere acceder a la columna
        """
        import types
        if indice.__class__ == types.IntType:
            return self._portero.actual().registros()[indice]
        else:
            return self.getCol(indice)
            
    def __setitem__(self, indice, valor): 
        """
            Modificacion por []
            Si el indice es un entero, se entiende que se 
            quiere modificar a un registro
            Si el indice es un nombre o el objeto variable, 
            se entiende que se quiere modificar a la columna
        """
        import types
        if indice.__class__ == types.IntType:
            self._portero.actual().registros[indice]=valor
        else: 
            #Añade los registros que no existen
            for registro in range(len(valor)-len(self._portero.actual().\
                    registros())):
                self.appReg()

            for registro in range(len(self._portero.actual().registros())):
                self._portero.actual().registros()[registro][indice] \
                        = valor[registro]

    def __delitem__(self, indice):
        """define el borrado con la función del, borra un registro"""
        import types
        if indice.__class__ == types.IntType:
            self._portero.actual().registros().__delitem__(indice)

    def borrartodo(self):
        """Borra todos los datos"""
        self._portero.nuevoestado(True)

    def getVar(self, indice=-1):
        """Devuelve la variable indicada por el indice, por defecto la última"""
        return self._portero.actual().variables()[self.obtenerIndiceVariable(indice)] 
    def numReg(self):
        """Devuelve el número de registros almacenados"""
        return len(self._portero.actual().registros())

    def numVar(self):
        """Devuelve el número de variables almacenadas"""
        return len(self._portero.actual().variables())

    def insReg(self, pos, dato = None):
        """inserta un registro en la posicion pos"""
        from datos import Registro
        import types
        if dato == None:
            registro = Registro(self._portero.actual().variables())
            self._portero.actual().registros().insert(pos, registro)
        elif isinstance(dato, Registro):
            self._portero.actual().registros().insert(pos, dato)
        elif dato.__class__ == types.ListType:
            registro = Registro(self._portero.actual().variables())
            registro.set(dato)
            self._portero.actual().registros().insert(pos, registro)
        else:
            raise TypeError

    def appReg(self,dato=None):
        pos=self.numReg()+1
        self.insReg(pos,dato)

    def appVar(self, nombre = None, tipo = "Entero", \
            valorpordefecto="NA", descripcion = "",protegerfiltro=True):  
        """
        Añade una variable de tipo tipo al array de variables. 
        Rellena los registros con el valor por defecto
        """
        if not nombre:
            if self.numVar()>0:
                nombreultimavariable=self.getVar(-1).nombre
                if nombreultimavariable.find("VAR")>-1:
                    numero=int(nombreultimavariable.replace("VAR",""))
                    nombre="VAR"+str(numero+1)
                else:
                    raise NameError
            else:
                nombre="VAR"+str(self.numVar())
        self._appVarPrivado(nombre,tipo,valorpordefecto,descripcion,protegerfiltro)
        for i in range(self.numReg()):
            self._portero.actual().registros()[i].append(valorpordefecto)

    def arrTitDiscreto(self):
        """Devuelve una lista con los titulos de las variables discretas"""
        lista=[]
        from agrupadores import AgrupadorDiscreto
        for variable in self._portero.actual().variables():
            if isinstance(variable,AgrupadorDiscreto):
                lista.append(variable.nombre)
        return lista

    def arrTitNumerica(self):
        """Devuelve una lista con los titulos de las variables discretas"""
        lista=[]
        from agrupadores import AgrupadorNumerico
        for variable in self._portero.actual().variables():
            if isinstance(variable,AgrupadorNumerico):
                lista.append(variable.nombre)
        return lista

    def arrTit(self, tipo = None, factores = True):
        """
        Devuelve una lista con todos los nombres de las variables
        opcionalmente podemos filtrar segun un tipo
        """
        if not tipo:
            if factores:
                return [variable.nombre for variable in \
                        self._portero.actual().variables()]
            else:
                lista=[]
                for variable in self._portero.actual().variables():
                    if variable.tipo!="Factor":
                        lista.append(variable.nombre)
                return lista

        else:
            lista=[]
            for variable in self._portero.actual().variables():
                if variable.tipo==tipo:
                    lista.append(variable.nombre)
            return lista
                
    def getCasos(self, indice = None):
        """Devuelve todos los casos existentes de una determinada variable"""
        #debe ser una variable discreta, si no excepcion
        lista=[]
        indiceentero = self.obtenerIndiceVariable(indice)
        for valor in self._portero.actual().registros():
            if valor[indiceentero] not in lista:
                lista.append(valor[indiceentero])
        lista.sort()
        return lista

    def getCol(self, indice):
        """Devuelve un array con todos los valores asociados a 
        la variable representada por indice.
        Para uso de las clases del programa
        """
        lista=[]
        indicenumerico=self.obtenerIndiceVariable(indice)
        for valor in self._portero.actual().registros():
            lista.append(valor[indicenumerico])
        return lista

    def insertarestado(self, estado, flagoriginal = False):
        """Inserta un estado tal cual"""
        self._portero.insertarestado(estado, flagoriginal = flagoriginal)

    def original(self):
        """Devuelve True o False en funcion de si el estado actual es original"""
        return self._portero.actualoriginal()

    def setFlagoriginal(self):
        """Establece la flag original"""
        self._portero.setactualoriginal()

    def delVar(self, variable):
        """Borra una variable y su contenido asociado"""
        indice = self.obtenerIndiceVariable(variable)
        for reg in self._portero.actual().registros():
            del reg[indice]
        del self._portero.actual().variables()[indice] 

    def _appVarPrivado(self,nombre,tipo,valorpordefecto,descripcion,protegerfiltro=True):
        """Añade una variable, sin preocuparse por los registros"""
        import agrupadores
        if nombre in self.arrTit(factores = True):
            from  Driza.excepciones import VariableExisteException
            raise VariableExisteException()
        if protegerfiltro:
            self._checkNombreFiltro(nombre)
        if tipo == "Entero":
            self._portero.actual().variables().append\
                    (agrupadores.AgrupadorEntero\
                    (nombre, valorpordefecto, descripcion)) 
        elif tipo == "Real":
            self._portero.actual().variables().append\
                    (agrupadores.AgrupadorReal\
                    (nombre, valorpordefecto, descripcion)) 
        elif tipo == "Factor":
            self._portero.actual().variables().append\
                    (agrupadores.AgrupadorFactor(nombre, descripcion)) 
        elif tipo == "Logico":
            self._portero.actual().variables().append\
                    (agrupadores.AgrupadorLogico(nombre, descripcion)) 
        else:
            raise TypeError #PENDIENTE: Averiguar el tipo de excepcion adecuado

    def _checkNombreFiltro(self,nombre):
        """Comprueba si el nombre no coincide con el nombre de filtro"""
        if nombre == "*filtro":
            #El nombre *filtro está reservado
            from  Driza.excepciones import VariableExisteException
            raise VariableExisteException()
    def obtenerIndiceVariable(self,variable):
        """Devuelve el indice numerico de la variable pasada"""
        return self._portero.actual().variables().obtenerindice(variable)

class InterfazDatosUsuario(InterfazDatos):
    """La interfaz de datos con el usuario"""
    def __init__(self, portero, gestorpaquetes):
        InterfazDatos.__init__(self,portero)
        self.__gestorpaquetes=gestorpaquetes

    def appVarExpresion(self, nombre, tipo, valorpordefecto, descripcion, expresion, vigilarfiltro=True,permitirsobreescritura=False,solofiltrados=False):  
        """
        Añade una variable de tipo tipo al array de variables. 
        Rellena los registros con el valor por defecto, 
        salvo que se indique una expresion
        """
        listaresultante = self.__interpretaexpresion(expresion)
        if solofiltrados:
            assert(permitirsobreescritura==True) #Si se trabaja solo sobre filtrados, es porque se permite sobreescritura
        if not permitirsobreescritura or not (nombre in self.arrTit(factores = True)):
            self._appVarPrivado(nombre,tipo,valorpordefecto,descripcion,vigilarfiltro)
            for i in range(self.numReg()):
                self._portero.actual().registros()[i].append(listaresultante[i])
        elif permitirsobreescritura and (nombre in self.arrTit(factores = True)):
            variable=self.getVar(nombre)
            if variable.tipo != tipo:
                raise TypeError
            self.modVar(variable,"valorpordefecto",valorpordefecto)
            self.modVar(variable,"descripcion",descripcion)
            posicion=self.obtenerIndiceVariable(nombre)
            if solofiltrados:
                indicefiltro=self.obtenerIndiceVariable("*filtro")
                for i in range(self.numReg()):
                    if self._portero.actual().registros()[i][indicefiltro]:
                        self._portero.actual().registros()[i][posicion]=listaresultante[i]
            else:
                for i in range(self.numReg()):
                    self._portero.actual().registros()[i][posicion]=listaresultante[i]

    def __interpretaexpresion(self,expresion):
        """Interpreta una expresion y devuelve una lista con los resultados"""
        log.debug("Interpretando expresion: "+expresion)
        listavars = []
        #Copia de la lista, ya que vamos a hacer un reverse
        listavars += self._portero.actual().variables() 
        #Evaluamos las variables con el nombre más grande para 
        #evitar los casos en los que una variable contenga a otra
        listavars.sort(lambda x, y: len(x.nombre) > len(y.nombre))
        listavars.reverse() 
        listaindicevariables=[] 
        for var in listavars:
            numero = self.obtenerIndiceVariable(var)
            if expresion.find(var.nombre)!=-1:
                listaindicevariables.append(numero)
                expresion = expresion.replace(var.nombre,"self.__getitem__(i)["+repr(numero)+"]")

        #lista = self.__gestorpaquetes.listafunciones()
        #cosatmp = self.__gestorpaquetes
        ##Recorremos la lista de funciones, y hacemos la sustitucion (Con prefijo)
        #for funcion in lista: 
        #    texto = string.split(funcion,".")
        #    ristra = "cosatmp[\""+texto[0]+"\"][\""+texto[1]+"\"]"
        #    expresion = expresion.replace(funcion, ristra)

        lista = self.__gestorpaquetes.listafunciones(False)
        cosatmp = self.__gestorpaquetes
        #Recorremos la lista de funciones, y hacemos la sustitucion (Sin prefijo)
        for funcion in lista: 
            ristra = "cosatmp[\""+funcion+"\"]"
            expresion = expresion.replace(funcion, ristra)
        lista=[]
        log.debug("Resultado expresion: "+expresion)
        for i in range(self.numReg()):
            valido=True
            for j in listaindicevariables:
                if not self.__getitem__(i)[j].valido():
                    valido=False
            if valido:
                lista.append(eval(expresion))
            else:
                lista.append(None)
        log.debug("Lista resultante de la expresion: "+str(lista))
        return lista

    def modVar(self, variable, indice, valor):
        """
        Modifica una variable existente. 
        Es necesario para verificar que el nombre no esta ya ocupado
        Variable es el objeto variable, no el indice
        """
        if indice==0 or indice=="nombre":
            if valor in self.arrTit(factores = True):
                from  Driza.excepciones import VariableExisteException
                raise VariableExisteException()
            self._checkNombreFiltro(valor)
            variable.nombre=valor
        elif indice==2 or indice=="valorpordefecto":
            variable.valorpordefecto=valor
        elif indice==3 or indice=="descripcion":
            variable.descripcion=valor

    def setVar(self, indice, variable, columna = None):
        """
        Sobreescribe la variable asociada al indice.
        Hace una conversión de tipos según la representación interna de estos
        Si se le pasa la columna, entonces sobreescribira todos 
        los registros asociados a la variable
        """
        #Debe comprobar la correctitud del array y lanzar 
        #una excepcion si no vale
        log.debug("Columna pasada a setVar: "+str(columna))
        array = []
        if variable.nombre in self.arrTit(factores = True) \
                and self.getVar(indice).nombre != variable.nombre:
            #Comprobamos que no se trata de un cambio de atributo 
            #de la variable
            from  Driza.excepciones import VariableExisteException
            raise VariableExisteException()
        self._checkNombreFiltro(variable.nombre)
        if not columna:
            #Conversion directa (Inestable)
            for elemento in self.getCol(indice):
                array.append(repr(elemento))
            self._portero.actual().variables()[indice] = variable
            for registro in self._portero.actual().registros():
                registro[indice] = variable.nuevoitem(array[indice])
        else:
            #Nos pasan la columna con los datos
            assert(len(columna)==self.numReg())
            self._portero.actual().variables()[indice] = variable
            i = 0
            for registro in self._portero.actual().registros():
                registro[indice] = variable.nuevoitem(columna[i])
                i += 1
        #TODO: Continuar con el caso de la columna

    def setFiltro(self,expresion):
        """Establece el valor del filtro y lo activa"""
        log.debug("Se solicita aplicar el filtro:"+expresion)
        listaresultante = self.__interpretaexpresion(expresion)
        if not "*filtro" in self.arrTit():
            self.appVar("*filtro","Logico",False,"None",False)
            i=0
            for registro in self._portero.actual().registros():
                registro[-1] = self.getVar(-1).nuevoitem(listaresultante[i])
                i+=1
        else:
            indicefiltro=self.obtenerIndiceVariable("*filtro")
            i=0
            for registro in self._portero.actual().registros():
                registro[indicefiltro] = self.getVar(indicefiltro).nuevoitem(listaresultante[i])
                i+=1
        #Calcular el valor para cada registro e introducirlo


    def delFiltro(self):
        """Borra/Desactiva el filtro"""
        if "*filtro" in self.arrTit():
            self.delVar("*filtro")


class InterfazDatosR(InterfazDatos):
    """La interfaz de datos con R"""
    def resolverEtiqueta(self,variable,caso):
        """Devuelve la etiqueta sie xiste, si no devuelve el caso tal cual"""
        variable=self.getVar(variable)
        #TODO... Convenir que tipos acepta y cuales no
        if variable.etiquetas.has_key(caso):
            return variable.etiquetas[caso]
        elif variable.etiquetas.has_key(str(caso)):
            return variable.etiquetas[str(caso)]
        else:
            return caso
    def RData(self):
        """Devuelve una copia de los datos actuales en el formato data de R"""
        #TODO: Falta el control de filtrado
        import rpy
        diccionario={}
        for variable in self._portero.actual().variables:
            if variable.tipo!="Factor":
                lista = map(float, self.__getitem__(variable))
            else:
                lista = map(str, self.__getitem__(variable))
            diccionario[variable.nombre]=lista
        df = rpy.r.data_frame(diccionario)
        return df

    def query(self, variable, *condiciones):
        """El query es una peticion de datos que sera enviada a R. 
        Hace prácticamente lo mismo que getCol, salvo que en este 
        caso la lista de datos es devuelta directamente en formato float, 
        y acepta condiciones de filtrado
        Para uso en las operaciones
        """
        from agrupadores import AgrupadorNumerico
        assert(isinstance(self.getVar(variable),AgrupadorNumerico))
        #TODO assert la variable de filtrado es distinta de la 
        #de alguna condicion
        #TODO assert la variable es distinta a la 
        #de alguna condicion
        from Driza.condiciones import parsecondicion
        mifuncion = lambda f: parsecondicion(self, f)
        listacondiciones = map(mifuncion, condiciones)
        indicenumerico=self.obtenerIndiceVariable(variable)
        lista=[]
        for valor in self._portero.actual().registros():
            esvalido = True
            campoactual=valor[indicenumerico]
            if not campoactual.valido():
                esvalido = False
            if ("*filtro" in self.arrTit()):
                indicefiltro=self.obtenerIndiceVariable("*filtro")
                from types import NoneType
                if isinstance(valor[indicefiltro].valor,NoneType):
                    esvalido = False
                elif not valor[indicefiltro]:
                    esvalido = False
            for condicion in listacondiciones:
                if not condicion(valor): 
                    #Tiene que verificar todas las condiciones
                    esvalido = False 
            if esvalido:
                lista.append(campoactual)
        lista=map(float,lista)
        return lista

class InterfazDatosFicheros(InterfazDatos):
    """La interfaz de datos con los ficheros"""
    def loadTxt(self,fichero,**opciones):
        import os
        if not os.path.exists(fichero):
            from Driza.excepciones import FicheroNoExisteException
            raise FicheroNoExisteException(fichero)
        cabeceras=opciones["cabeceras"]
        delimitadoratrib=opciones["delimitadoratrib"]
        from rpy import r
        datos = r.read_table(fichero, sep = delimitadoratrib, \
                header = cabeceras, na_strings="NA", dec=".", \
                strip_white = False)
        for pareja in datos.items():
            import types
            milista=[]
            if type(pareja[1][0])==types.IntType:
                self.appVar(pareja[0],"Entero")
                for x in pareja[1]:
                    if x==-2147483648:
                        resultado=float('nan')
                    else: resultado = x
                    milista.append(resultado)
            elif type(pareja[1][0])==types.StringType:
                #Es un Factor
                self.appVar(pareja[0],"Factor")
                milista=pareja[1]
            elif type(pareja[1][0])==types.FloatType:
                self.appVar(pareja[0],"Real")
                milista=pareja[1]
            else: print "ERROR",pareja[1][0] #TODO: Cambiar por excepcion

            self.__setitem__(self.getVar(pareja[0]),milista)

