#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007  Nstor Arocha Rodrguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Definicion de las clases que representan variables"""

class Variable:
    """La clase padre de todas las variables"""
    def __init__(self,agrupador,valor):
        self.tvariable = agrupador
        self.valor = self._interno(valor)

    def valido(self):
        if self.valor==None: 
            return False
        return True

    def __cmp__(self,otro):
        if self.valor==None:
            return False
        if isinstance(otro,Variable):
            if otro.valor==None:
                return False
            else:
                return self.valor.__cmp__(otro.valor)
        else:
            return self.valor.__cmp__(otro) #Delegamos la excepcion a la clase de otro

    def __repr__(self):
        return repr(self.valor)

    def __str__(self):
        if self.valor==None:
            return "NA"
        return str(self.valor)

    def etiqueta(self):
        """Devuelve la representacin con etiqueta de la variable
        Las clases hijas que esten sujetas a etiquetado redefinen este metodo"""
        return self.__str__()

class VariableNumerica:
    """Las clases que heredan son numericas. Esta clase solo aporta esa informacion de momento"""
    def __add__(self,otro):
        if self.valor==None:
            return None
        return self.valor + int(float(otro))

    def __eq__(self,otro):
        if self.valor==None:
            return False
        return self.valor.__eq__(float(otro))

    def __pow__(self,exponente):
        if self.valido():
            if isinstance(exponente,VariableNumerica):
                return self.valor.__pow__(exponente.valor)
            return self.valor.__pow__(exponente)
        return float('nan')

    def __rpow__(self,base):
        if self.valido():
            if isinstance(base,VariableNumerica):
                return self.valor.__rpow__(base.valor)
            return self.valor.__rpow__(base)
        return float('nan')


class VariableDiscreta(Variable):
    """Clase que representa a las variables discretas. Permiten la comparacion directa entre valores"""
    def __eq__(self,other):
        if isinstance(other,Variable):
            return self.valor==other.valor
        else:
            return self.valor==other

class VariableContinua(Variable):
    """Representa a las variables continuas"""
    def __float__(self):
        return 0

class Factor(VariableDiscreta):
    """Las variables que solo son cadenas y que nicamente sirven para agrupacin"""
    def _interno(self,cadena):
        if cadena=="''":
            return str()
        if isinstance(cadena,str):
            return cadena
        else:
            return str(cadena)

    def __str__(self):
        if self.valor!=None:
            return self.valor
        return ''

    def __repr__(self):
        if self.valor!=None:
            return self.valor
        return ''
    
    def __cmp__(self,otrovalor):
        if self.valor==None:
            return False
        if isinstance(otrovalor,Variable):
            if otrovalor.valor==None:
                return False
            else:
                return self.__compare(otrovalor.valor)
        else:
            return self.__compare(otrovalor) #Delegamos la excepcion a la clase de otro

    def __compare(self,otrovalor):
        if self.valor < otrovalor: return -1
        elif self.valor > otrovalor: return 1
        else: return 0


class Real(VariableContinua,VariableNumerica):
    def _interno(self,valor):
        if isinstance(valor,Real):
            return valor.valor
        elif valor==None or valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        elif isinstance(valor,float):
            return valor
        else:
            return float(valor)

    def __float__(self):
        if self.valido():
            return self.valor
        else:
            return float("nan")

    def __cmp__(self,otrovalor):
        if self.valor==None:
            return False
        if isinstance(otrovalor,Variable):
            if otrovalor.valor==None:
                return False
            else:
                return self.__compare(otrovalor.valor)
        else:
            return self.__compare(otrovalor) #Delegamos la excepcion a float

    def __compare(self,valor):
        """Hace el equivalente de comparacion para el float"""
        if self.valor>valor:
            return 1
        elif self.valor<valor:
            return -1
        else: return 0

    def __str__(self):
        if self.valor==None:
            return "NA"
        #ristra ="%8."+str(self.tvariable.numerodecimales) +"g"
        decimales=str(self.valor).split(".")[1]
        if len(str(decimales))>self.tvariable.numerodecimales:
            ristra ="%8."+str(self.tvariable.numerodecimales) +"f"
            return ristra % self.valor
        return str(self.valor)


class Entero(VariableDiscreta,VariableNumerica):
    def _interno(self,valor):
        if valor == None or valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        elif isinstance(valor,int):
            return valor
        else:
            return int(valor)
    def __float__(self):
        if self.valido():
            return float(self.valor)
        else:
            return float('nan')
    def __int__(self):
        if self.valido():
            return int(self.valor)
        else:
            return None
    def etiqueta(self):
        if self.valido():
            if self.tvariable.etiquetas.has_key(str(self.valor)):
                return self.tvariable.etiquetas[str(self.valor)]
        return self.__str__()

 
class Ordinal(VariableDiscreta):
    def _interno(self,valor):
        if valor == None or valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        elif isinstance(valor,Ordinal):
            return valor.valor
        elif isinstance(valor,int):
            return valor
        else:
            return int(valor)
    def __float__(self):
        if self.valido():
            return float(self.valor)
        else:
            return float('nan')

    def etiqueta(self):
        if self.valido():
            if self.tvariable.etiquetas.has_key(str(self.valor)):
                return self.tvariable.etiquetas[str(self.valor)]
        return self.__str__()

class Logico(VariableDiscreta):
    def _interno(self,valor):
        if valor == None or valor=="None" or valor=="NA" or valor=="nan" or valor==float("nan"):
            return None
        elif valor == True or valor == "True" or valor == "Verdadero" or valor =="1":
            return True
        else:
            return False

    def __nonzero__(self):
        if self.valor == None: return False
        return self.valor.__nonzero__()

    def __eq__(self,otro):
        if self.valor==None:
            return False
        return self.valor.__eq__(otro.valor)

    def __xor__(self,otro):
        if self.valor==None:
            return False
        return self.valor.__xor__(otro.valor)

    def __int__(self):
        if self.valor==None:
            return None
        return int(self.valor)

    def __cmp__(self,otro):
        if self.valor==None:
            return False
        return self.valor.__cmp__(otro.valor)

    def __str__(self):
        if self.valor==None: return "NA"
        elif self.valor==True: return "1"
        elif self.valor==False: return "0"
