#!/usr/bin/python
# -*- coding: utf-8 -*-

#Dialogo de creacion de variables


#Copyright (C) 2006-2007  Néstor Arocha Rodríguez,Inmaculada Luengo Merino 

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#pushButton1: Boton aceptar
#pushButton2: Cancelar
#pushButton3: Boton de inclusion de funciones en caja de texto
#pushButton4 Inclusión de variable en caja de texto
#textEdit1: Caja de escritura
#listbox1: listado de variables
#listbox2: listado categoria funciones
#listbox3: listado funciones
#lineEdit1: Caja con el nombre de la variable
#lineEdit2: Caja con la descripción
#comboBox1: Caja con el tipo


from Driza.iuqt3.ui.dcrevar import DialogoCrevar
from Driza.funciones import GestorPaquetes
from qt import *
import logging
log = logging.getLogger("Driza.iuqt3.dcrevar")


class DCrevar(DialogoCrevar):
    """Dialogo de creación de variables, permite obtener nuevas variables a partir de las ya existentes"""

    def __init__(self,parent,dato,config,gestorpaquetes):
        """Inicialización"""
        DialogoCrevar.__init__(self,parent,"dialogo Creacion variables",0,0)
        #VARIABLES PRIVADAS
        self.__gestorpaquetes = gestorpaquetes
        self.__config=config
        self.__dato=dato
        self.checkBox2.setDisabled(True)

        self.__inicializarlistatipos()
        self.__conexiones()
    #FUNCIONES PUBLICAS        

    def showEvent(self,ev):
        self.__actualizarcajas()
        DialogoCrevar.showEvent(self,ev)
    
    def accept(self):
        """Función de aceptación del dialogo. Avisa al usuario si los datos han sido introducidos incorrectamente, y genera"""
        # Creo un objeto con lo que me pasan a ver si hay una excepción o no
        if self.lineEdit1.text():
            from Driza.excepciones import VariableExisteException
            sobreescritura=self.checkBox1.isChecked()
            solofiltrado=False
            if sobreescritura:
                if self.checkBox2.isChecked():
                    solofiltrado=True
            try:
                self.__dato.appVarExpresion(self.lineEdit1.text().latin1(),self.comboBox1.currentText(),0,self.lineEdit2.text(),self.textEdit1.text().latin1(),permitirsobreescritura=sobreescritura,solofiltrados=solofiltrado)
            except (SyntaxError,NameError):
                #La expresión no es correcta, mostramos un mensaje de error
                log.exception("Excepción al añadir la variable con la expresión")
                QErrorMessage(self,"error").message(u"La expresión no es correcta")
                self.__dato.delVar(self.lineEdit1.text().latin1()) #Borrar la variable que está a medias
            except (ZeroDivisionError,OverflowError):
                QErrorMessage(self,"error").message(u"La expresión genera un desbordamiento")
            except VariableExisteException:
                QMessageBox.warning(self,u"atención",u"La variable ya existe")
            except TypeError:
                QErrorMessage(self,"error").message(u"El tipo de variable no coincide con el existente")
                log.exception("Excepción al añadir la variable con la expresión")
            except:
                QErrorMessage(self,"error").message(u"Error desconocido")
                log.exception("Error no gestionado")

            else:
                DialogoCrevar.accept(self)
                self.parent().grid.myUpdate()
        else:
            rc=QMessageBox.warning(self,'Atencion', 'No has rellenado todos los campos', 'Volver al dialogo','Salir','',0,1 )
            if rc==1:
                self.reject()


    #FUNCIONES PRIVADAS
    def __conexiones(self):
        """Bloque de conexiones"""
        self.connect(self.pushButton1,SIGNAL("clicked()"),self.accept)
        self.connect(self.pushButton2,SIGNAL("clicked()"),self.reject)
        self.connect(self.pushButton3,SIGNAL("clicked()"),self.__insertarfuncionseleccionada)
        self.connect(self.pushButton4,SIGNAL("clicked()"),self.__insertarvariableseleccionada)
        self.connect(self.comboBox1,SIGNAL("activated(int)"),self.__actualizarcajas)
        self.connect(self.listBox2,SIGNAL("selectionChanged()"),self.__actualizarlistafunciones)
        self.connect(self.pushButton5,SIGNAL("clicked()"),self.__insertarSimboloMas)
        self.connect(self.pushButton6,SIGNAL("clicked()"),self.__insertarSimboloMenos)
        self.connect(self.pushButton7,SIGNAL("clicked()"),self.__insertarSimboloAsterisco)
        self.connect(self.pushButton8,SIGNAL("clicked()"),self.__insertarSimboloBarra)
        self.connect(self.pushButton9,SIGNAL("clicked()"),self.__insertarSimboloDobleAsterisco)
        self.connect(self.checkBox1,SIGNAL("clicked()"),self.__activarcheckbox2)

    def __actualizarcajas(self):
        """Actualiza todas las cajas"""
        self.__actualizarcajavars()
        self.__actualizartipofunciones()


    def __actualizarcajavars(self):
        """Actualiza la caja de variables disponibles"""
        self.listBox1.clear()
        i=0
        j=0
        for var in self.__dato.arrTit():
            if cmp(self.comboBox1.currentText().latin1(),self.__dato.getVar(j).tipo)==0:
                self.listBox1.insertItem(var,i)
                i+=1
            j+=1
    
    def __inicializarlistatipos(self):
        self.comboBox1.clear()
        from Driza import listas
        for tipo in listas.variables():
            self.comboBox1.insertItem(tipo)

    def __actualizartipofunciones(self):
        """Actualiza la lista de tipo de funciones en funcion del tipo destino"""
        self.listBox2.clear()
        for x in self.__gestorpaquetes.iterkeys():
            self.listBox2.insertItem(x)
    
    def __insertarvariableseleccionada(self): 
        """Coge el valor seleccionado y lo añade en la caja de texto"""
        if self.listBox1.currentItem() != -1:
            texto=self.listBox1.currentText().latin1()
            self.textEdit1.insert(texto)

    def __insertarfuncionseleccionada(self):
        """Inserta la funcion seleccionada en la caja de texto"""
        if self.listBox3.currentItem() != -1:
            texto=self.listBox3.currentText().latin1()
            self.textEdit1.insert(texto)

    def __actualizarlistafunciones(self):
        self.listBox3.clear()
        if cmp(self.listBox2.currentText(),u"Aritméticas")==0:
            #Estamos con las aritmeticas
            self.listBox3.insertItem("+")
            self.listBox3.insertItem("-")
            self.listBox3.insertItem("*")
            self.listBox3.insertItem("/")
        elif cmp(self.listBox2.currentText(),u"Lógicas")==0:
            self.listBox3.insertItem("and")
            self.listBox3.insertItem("or")
        else:
            lista=self.__gestorpaquetes[unicode(self.listBox2.currentText().latin1(),'iso-8859-1')].values()
            for x in lista:
                self.listBox3.insertItem(x.nombre)

    def __insertarSimboloMas(self): self.textEdit1.insert("+")
    def __insertarSimboloMenos(self): self.textEdit1.insert("-")
    def __insertarSimboloAsterisco(self): self.textEdit1.insert("*")
    def __insertarSimboloBarra(self): self.textEdit1.insert("/")
    def __insertarSimboloDobleAsterisco(self): self.textEdit1.insert("**")

    def __activarcheckbox2(self):
        if self.checkBox1.isChecked(): self.checkBox2.setEnabled(True)
        else: self.checkBox2.setDisabled(True)
