#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from Driza.iuqt3.ui.doperaciones import DialogoOperaciones
from Driza.operaciones import GestorMensajesCategorias
import logging
log = logging.getLogger("Driza.iuqt3.operaciones.doperaciones")

class DOperaciones(DialogoOperaciones):
    """Dialogo  de operaciones. Muestra las operaciones disponibles
    Y ademas muestra el woperaciones de la operación actualmente seleccionada
    """

    def __init__(self,parent,idu,gestoroperaciones,vsalida):
        self.__diccionariowidgets={}
        self.gestorcategorias=GestorMensajesCategorias()
        DialogoOperaciones.__init__(self,parent,"Dialogo casos",0,0)
        self.__idu=idu
        self.__gestoroperaciones=gestoroperaciones
        self.__initwidgets()
        self.__initarbol()
        self.__conexiones()
        self.__vsalida=vsalida
        self.__noimplementado=QLabel(None,"No implementado")
        self.__noimplementado.setText("No implementado")
        self.__condicionesincorrectas=QLabel(None,"No se dan las condiciones adecuadas")
        self.__condicionesincorrectas.setText("No se dan las condiciones adecuadas")
        self.widgetStack1.addWidget(self.__condicionesincorrectas)
        self.listView2.setRootIsDecorated(True)
        self.listView2.addColumn("Operaciones")
        self.__operacion=None #Guarda la operacion actual
    
    def mostrar(self,operacion):
        self.__operacion=operacion
        self.show()

    def showEvent(self,ev):
        self.__myUpdate()
        DialogoOperaciones.showEvent(self,ev)

    def accept(self):
        """Acepta el dialogo. Coge la operacion con la que esta trabajando el usuario e inicia el procedimiento de calculo"""
        from Driza.excepciones import OpcionesIncorrectaException
        import rpy
        nombre=unicode(self.listView2.currentItem().text(0).latin1(),'iso-8859-1')
        widget=self.__diccionariowidgets[nombre]
        try:
            resultado=self.__gestoroperaciones[nombre].procedimiento(widget.seleccion(),widget.opciones())
        except OpcionesIncorrectaException:
            QErrorMessage(self,"error").message(u"Las opciones no son correctas")
        except IndexError:
            QErrorMessage(self,"error").message(u"Seleccion Incorrecta")
            log.exception("excepcion capturada")
        except KeyError:
            QErrorMessage(self,"error").message(u"Hay elementos de la salida sin definir(o mal definidos) en la operación")
            log.exception("Excepción Generada por un módulo de operaciones")
        except rpy.RException:
            QErrorMessage(self,"error").message(u"R devolvio un error")
            log.exception("Excepción de RPY")
        except AssertionError:
            QErrorMessage(self,"error").message(u"Error desconocido")
        else:
            self.__vsalida.appRes(resultado) #Añadir a la salida el resultado
            self.__vsalida.hide() #TODO averiguar como hacer para que recupere el foco sin ocultar la ventana
            self.__vsalida.show()

    def __cambiarelemento(self):
        self.__operacion=unicode(self.listView2.currentItem().text(0).latin1(),'iso-8859-1')
        self.__showwidget()


    def __initwidgets(self):
        """Añade los widgets en el widgetstack y en el diccionario de widgets"""
        for titulo,operacion in self.__gestoroperaciones.items():
            widget=self.__renderwidget(titulo,operacion.widget)
            self.__diccionariowidgets[titulo]=widget
            self.widgetStack1.addWidget(widget)

    def __initarbol(self):
        """Inicializa el arbol con todas las operaciones conocidas
        """
        #TODO En futuras implementaciones la lista se generará dinámicamente
        self.listView2.clear()
        descriptivo=QListViewItem(self.listView2,"Descriptivos")
        self.gestorcategorias["Descriptivos"]="En esta categoria se encuentran los mensajes descriptivos"
        self.widgetStack1.addWidget(self.gestorcategorias["Descriptivos"])
        estadisticos=QListViewItem(descriptivo,u"Estadísticos")
        QListViewItem(estadisticos,"Descriptivo")
        QListViewItem(descriptivo,u"Tabla de frecuencias")
        graficos=QListViewItem(descriptivo,u"Gráficas")
        QListViewItem(graficos,"Histograma")
        QListViewItem(graficos,u"Diagrama de dispersión")
        QListViewItem(graficos,u"Diagrama de barras")

        cmedias=QListViewItem(self.listView2,u"Comparación de medias")
        QListViewItem(cmedias,u"Comparación de medias simple")
        QListViewItem(cmedias,u"Media de dos muestras independientes")
        QListViewItem(cmedias,u"Anova Simple")
        cprop=QListViewItem(self.listView2,u"Comparación de proporciones")
        QListViewItem(cprop,u"Comparación de proporciones simple")
        cvar=QListViewItem(self.listView2,u"Comparación de Varianzas")
        QListViewItem(cvar,u"Comparación de Varianzas entre dos variables")
        QListViewItem(cvar,u"Comparación de Varianzas entre múltiples variables")
        bajuste=QListViewItem(self.listView2,"Bondad de ajuste")
        QListViewItem(bajuste,"KS")
        QListViewItem(bajuste,"Ji Cuadrado")
        rlineal=QListViewItem(self.listView2,u"Regresión Lineal")
        QListViewItem(rlineal,u"Regresión")
        distribuciones=QListViewItem(self.listView2,"Distribuciones")
        QListViewItem(distribuciones,u"Distribución Normal")

    def __conexiones(self):
        self.connect(self.listView2,SIGNAL("selectionChanged ()"),self.__cambiarelemento)
        self.connect(self.pushButton2,SIGNAL("clicked()"),self.reject)
        self.connect(self.pushButton1,SIGNAL("clicked()"),self.accept)

    def __myUpdate(self):
        self.__showarbol()
        self.__showwidget()

    def __showarbol(self):
        """Muestra el arbol segun el contenido de gestoroperaciones"""
        #FIXME No deberia ser rellenado más de una vez
        self.listView2.ensureItemVisible(self.listView2.findItem(self.__operacion,0))
        self.listView2.setCurrentItem(self.listView2.findItem(self.__operacion,0))

    def __showwidget(self):
        """Muestra el widget asociado a la seleccion actual"""
        nombre=unicode(self.listView2.currentItem().text(0).latin1(),'iso-8859-1')
        log.debug('mostrando widget:'+nombre)
        if self.__gestoroperaciones.has_key(nombre):
            if self.__gestoroperaciones.funcionchequeocondiciones(nombre):
                self.widgetStack1.raiseWidget(self.__diccionariowidgets[nombre])
            else:
                self.widgetStack1.raiseWidget(self.__condicionesincorrectas)
        elif self.gestorcategorias.has_key(nombre):
            self.widgetStack1.raiseWidget(self.gestorcategorias[nombre])
        else:
            self.widgetStack1.raiseWidget(self.__noimplementado)

    def __renderwidget(self,nombre,diccionariowidget):
        """Renderiza un widget a partir de una descripcion en dicccionario"""
        import woperaciones
        if diccionariowidget["tipo"]=="Variable":
            widget=woperaciones.WidgetVariable(nombre,diccionariowidget["opciones"],self.__idu)
        elif diccionariowidget["tipo"]=="BondadDeAjuste":
            widget=woperaciones.WidgetCalculoSimpleBondadAjuste(nombre,diccionariowidget["listaoperaciones"],datos)
        elif diccionariowidget["tipo"]=="CalculoDoble":
            widget=woperaciones.WidgetCalculoDoble(nombre,diccionariowidget["opciones"],self.__idu)
        elif diccionariowidget["tipo"]=="ProporcionSimple":
            widget=woperaciones.WidgetCaso(nombre,diccionariowidget["opciones"],datos)
        elif diccionariowidget["tipo"]=="Graficos":
            widget=woperaciones.WidgetGraficos(nombre,diccionariowidget["graficos"],diccionariowidget["opciones"],datos)
        elif diccionariowidget["tipo"]=="Distribucion":
            widget=woperaciones.WidgetDistribucion(nombre,diccionariowidget["opcionescomun"],[],self.__idu)
        elif diccionariowidget["tipo"]=="2Variables3Casos":
            widget=woperaciones.Widget2Variable3Casos(nombre,diccionariowidget["opciones"],self.__idu)
        elif diccionariowidget["tipo"]=="VariableCasoCaso":
            widget=woperaciones.WidgetVariableCasoCaso(nombre,diccionariowidget["opciones"],self.__idu)
        elif diccionariowidget["tipo"]=="VariableCasoCasoVariableVariable":
            widget=woperaciones.WidgetVariableCasoCasoVariableVariable(nombre,diccionariowidget["opciones"],self.__idu)
        elif diccionariowidget["tipo"]=="DiscriminadorCasoCasoVariable":
            widget=woperaciones.WidgetDiscriminadorCasoCasoVariable(nombre,diccionariowidget["opciones"],self.__idu)
        else:
            raise NameError
        return widget
