#!/usr/bin/python
# -*- coding: utf-8 -*-
#

#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""woperaciones.py: Todos los widgets de operaciones"""

from qt import *

class WidgetOperacion(QWidget):
    """Clase base de todos los widgets de operaciones"""
    def __init__(self,name):
        if isinstance(name,unicode):
            name=name.encode("latin1")
        if not name:
            self.setName("WidgetOperacion")
        QWidget.__init__(self,None,name,0)
        self._rejilla=QGridLayout(self,1,1,11,6,"rejilla")
        self.languageChange()
        self.resize(QSize(600,480).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)


    def languageChange(self):
        self.setCaption(self.__tr("Form1"))

    def __tr(self,s,c = None):
        return qApp.translate("Form1",s,c)
    
    def seleccion(self):
        return self._wseleccion.seleccion

    def opciones(self):
        return self._wopciones.opciones()

class WidgetOperacionSelectorOpcion(WidgetOperacion):
    """WidgetOperacionSelectorOpcion es un widget de operaciones que tiene un selector y un widget de opciones"""
    def __init__(self,name,widgetselector,widgetopciones):
        WidgetOperacion.__init__(self,name)

        #Condiciones a probar
        from wopciones import WOpciones
        from seleccion import SelectorElementosEstudio
        assert(isinstance(widgetselector,SelectorElementosEstudio))
        assert(isinstance(widgetopciones,WOpciones))

        #Inicializacion de widgets
        self._wseleccion=widgetselector
        self._wopciones=widgetopciones

        zonastacksuperior=QWidgetStack(self,"stacksuperior")
        zonastacksuperior.setMinimumSize(QSize(200,200))
        zonastacksuperior.addWidget(self._wseleccion)
        zonastacksuperior.raiseWidget(self._wseleccion)
        zonastackinferior=QWidgetStack(self,"stackinferior")
        zonastackinferior.setMinimumSize(QSize(200,200))
        zonastackinferior.addWidget(self._wopciones)
        zonastackinferior.raiseWidget(self._wopciones)

        self._rejilla.addWidget(zonastacksuperior,0,0)
        self._rejilla.addWidget(zonastackinferior,1,0)

class WidgetVariable(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector simple y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariable(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

class WidgetCalculoSimpleBondadAjuste(WidgetVariable):
    """Es un widget especifico para todas las operaciones de bondad de ajuste"""
    def __init__(self,name,listadistribuciones,interfazdatos):
        listawidget=[]
        listawidget.append({"tipo":"SeleccionSimple","nombre":u"Distribución","opciones":listadistribuciones})
        listawidget.append({"tipo":"ListaSimple","nombre":u"Percentil","opciones":["95","90"]})
        WidgetVariable.__init__(self,name,listawidget,interfazdatos)
        
class WidgetVariableVariable(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector doble y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableVariable(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)


class WidgetVariableCaso(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableValor(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

class WidgetVariableVariableCaso(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from Driza.widgets.wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableVariableValor(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

class WidgetVariableCasoCaso(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from Driza.widgets.wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableValorValor(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

class WidgetVariableCasoCasoVariableVariable(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from Driza.widgets.wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableValorValorVariableVariable(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

class WidgetDiscriminadorCasoCasoVariable(WidgetOperacionSelectorOpcion):
    """Es un widget de calculo, con un selector variablevalor  y un widget de opciones suministrado en el constructor"""
    def __init__(self,name,listaopciones,interfazdatos):
        from wopciones import WOpciones
        import seleccion
        wseleccion=seleccion.SelectorVariableValorValorDiscriminador(interfazdatos)
        wopciones=WOpciones(interfazdatos,listaopciones)
        WidgetOperacionSelectorOpcion.__init__(self,name,wseleccion,wopciones)

