#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007   Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Grid y clases asociadas"""

from qt import *
from qttable import *
from Driza.datos.variables import * #Para poder preguntar por todos los tipos
import logging
log = logging.getLogger("Driza.iu.widgets.grid")

#http://www.digitalfanatics.org/projects/qt_tutorial/chapter13.html
#http://www.handhelds.org/~zecke/apidocs/libopie/html/ocheckitem_8cpp-source.html
#Mirar tablestatistics.py de ejemplos de PyQT
#http://lists.trolltech.com/qt-interest/2005-10/thread00469-0.html

class ButtonTableItem(QTableItem):
    """Esta clase hereda de QTableItem. Modifica dos aspectos de estos:
    	Por un lado inhabilita el texto que muestra normalmente
    	Por otro, utiliza un botón para que al ser pulsado lanza el dialogo de casos
    	"""

    def __init__(self, tabla, dcasos):
    	"""la inicialización llama al padre"""
    	QTableItem.__init__(self, tabla, QTableItem.WhenCurrent, "...") 
    	self.setReplaceable(False)
    	self.__dcasos=dcasos

    #FUNCIONES PUBLICAS
    	
    def createEditor(self):
    	"""Crea un editor para interactuar con el contenido.
    	En este caso es un botón, ya que permite asignar una acción al click de ratón"""
    	self.boton=QButton(self.table().viewport(),"Ventana abierta") #El padre es viewport, es invisible
        QObject.connect(self.boton,SIGNAL("clicked()"),self.__editar)
    	#QObject.connect(self.boton,SIGNAL("clicked()"),self.table(), SLOT("doValueChanged()"))
        return self.boton

    def setContentFromEditor(self,widget):
    	"""Cuando el contenido de widget cambia, qt llama a esta función.
    	Esta desactivada ya que el botón que actua de editor no dispone de información para devolver"""
        if isinstance(widget,QButton):
            self.setText("...")
        else:
            QTableItem.setContentFromEditor(self,widget)

    #FUNCIONES PRIVADAS
    def __editar(self):
    	"""Abre el dialogo de casos, pasando la columna como valor"""
    	self.__dcasos.mostrar(self.row())



class  Grid(QTabWidget):
    """La tabla que manejara los registros y las variables"""

    #Funciones de inicializacion
    def __init__(self,padre,interfazdatos,porterodatos):
        """Inicializacion"""
        from Driza.datos.conversion import AgenteConversion
        self.__portero=porterodatos
        QTabWidget.__init__(self,padre,"Grid")
        self.setTabPosition(QTabWidget.Bottom)
        self.tab = QWidget(self,"tab")
        tabLayout = QVBoxLayout(self.tab,11,6,"tabLayout")
        self.table1 = QTable(self.tab,"table1")
        tabLayout.addWidget(self.table1)
        self.insertTab(self.tab,QString.fromLatin1("&Registros"))
        self.tab_2 = QWidget(self,"tab_2")
        tabLayout_2 = QVBoxLayout(self.tab_2,11,6,"tabLayout_2")
        self.table2 = QTable(self.tab_2,"table2")
        tabLayout_2.addWidget(self.table2)
        self.insertTab(self.tab_2,QString.fromLatin1("&Variables"))
        from dcasos import DCasos
        self.dcasos=DCasos(self.table1,interfazdatos)
        self.modoetiqueta=False # Variable que guarda si estamos en modo etiqueta o normal

        self.__nreg=10 
        self.__nvar=10
        self.__idu=interfazdatos
        self.__inittabladat()
        self.__inittablavar()
        self.__conexiones()
        self.__agenteconversion=AgenteConversion(self.__idu)
        

    def __inittabladat(self):
        """Inicializa la tabla de datos"""
        self.table1.setNumCols(16) 
        self.table1.setNumRows(64)
        i = 0
        for x in range(self.table1.horizontalHeader().count()):
            self.table1.horizontalHeader().setLabel(i, '')
            i += 1


    def __inittablavar(self):
        """Inicializa la tabla de variables"""
        self.table2.setSelectionMode(QTable.MultiRow)
        self.table2.setNumRows(self.__nvar)
        self.table2.setNumCols(5)
        titulos=self.table2.horizontalHeader()
        titulos.setLabel (0,"Nombre")
        titulos.setLabel (1,"Tipo")
        titulos.setLabel (2,"Valor por defecto")
        titulos.setLabel (3,u"Etiqueta")
        titulos.setLabel (4,u"Etiquetas de valor")

    def __conexiones(self):
        self.connect(self.table1,SIGNAL("valueChanged(int,int)"),self.__modificaciontablareg)
        self.connect(self.table2,SIGNAL("valueChanged(int,int)"),self.__modificaciontablavar)

    
    #FUNCIONES SHOW

    def myUpdate(self):
        log.debug("Actualizando contenido de la ventana")
        self.__showtablavar()
        self.__showtabladat()



    def showEvent(self,ev):
        """Redefinición del show de la clase base"""
        self.myUpdate()
        QTabWidget.showEvent(self,ev)
        

    def __showtablavar(self,pos=None):  
        """ Representa la variable pos en la tabla de variables. Las propiedades de la variable las lee de los datos.
        Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
        """
        if pos:
            self.__showvar(pos)
            #Actualizacion de titulo en la tabla de datos
            self.__showtitulodat(pos)
        else:
            log.debug("Actualizando tabla de variables completa")
            if self.__nvar>(self.table2.numRows()-self.__idu.numVar()):
                self.table2.setNumRows(self.__idu.numVar()+self.__nvar)
            for n in range(self.__idu.numVar(),self.table2.numRows()):
                for y in range(self.table2.numCols()):
                    self.table2.clearCell(n,y)
            for n in range(self.__idu.numVar()):
                self.__showvar(n)
            self.__showtitulodat()

    def __showvar(self,pos):
        """Muestra una unica variable (fila) en la tabla segun el listado de variables"""
        variable=self.__idu.getVar(pos)
        self.table2.setText(pos,0,str(variable.nombre))
        self.table2.setItem(pos,1,self.__combotableitem())    
        self.table2.item(pos,1).setCurrentItem(variable.tipo)
        self.table2.setText(pos,3,str(variable.descripcion))
        #self.table2.setItem(pos,2,widget)
        self.table2.setText(pos,2,str(variable.valorpordefecto))
        self.table2.setItem(pos,4,self.__botontableitem())    

    def __showtabladat(self,pos=None):
        """Muestra el dato en la posicion pos (fila)
        Lee de los datos, así que deben existir previamente
        Comprueba si esta filtrado o no para cambiar el color
        Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
        """
        if pos:  
            #Se ha indicado la posición
            self.__showdat(pos)
        else: 
            #No se ha indicado la posicion, se rellena todo
            if self.__nreg>(self.table1.numRows()-self.__idu.numReg()):
                self.table1.setNumRows(self.__idu.numReg()+self.__nreg)
            for x in range(self.__idu.numReg()):
                self.__showdat(x)
            self.__showtitulodat()
            for x in range(self.__idu.numReg(),self.table1.numRows()):
                for y in range(self.table1.numCols()):
                    self.table1.clearCell(x,y)
            for x in range(self.table1.numRows()):
                for y in range(self.__idu.numVar(),self.table1.numCols()):
                    self.table1.clearCell(x,y)
            self.__showlateraldat()

    def __showdat(self,pos):
        """Muestra un solo dato"""
        if self.modoetiqueta:
            for i in range(self.__idu.numVar()):
                self.table1.setText(pos,i,self.__idu[pos][i].etiqueta())
        else:
            for i in range(self.__idu.numVar()):
                self.table1.setText(pos,i,str(self.__idu[pos][i]))

    def __showColumnaTablaReg(self,pos):
        """Muestra una columna de la tabla de registros"""
        for i in range(self.__idu.numReg()):
            self.table1.setText(i,pos,str(self.__idu[i][pos]))


    def __showtitulodat(self, pos=None):
        """Actualiza los titulos de la tabla de datos segun las variables"""
        titulos=self.table1.horizontalHeader()
        if pos:
            titulos.setLabel(pos, self.__idu.getVar(pos).nombre)
        else:
            i = 0
            for x in range(self.__idu.numVar()):
                self.table1.horizontalHeader().setLabel(i,self.__idu.getVar(i).nombre)
                i += 1

            for x in range(self.__idu.numVar(),self.table1.horizontalHeader().count()):
                self.table1.horizontalHeader().setLabel(i,'')
                i += 1

    def __showlateraldat(self):
        """Muestra los numeros laterales. Sirve para el filtrado"""
        lateral=self.table1.verticalHeader()
        #lista=self.__idu.getCol(self.__gestorfiltro.variable,filtrado=False)
        for i in range(self.__idu.numReg()):
        #    if self.__gestorfiltro.variable and not lista[i]:
        #        lateral.setLabel(i,"--"+str(i))
        #    else:
            lateral.setLabel(i,str(i+1))

    def __combotableitem(self):
        """Devuelve un nuevo objeto tipo combotableitem con la lista de tipos"""
        lista=QStringList()
        from Driza import listas
        for tipo in listas.variables():
            lista.append(tipo)
        return QComboTableItem(self.table2,lista)
    
    def __botontableitem(self):
        """Devuelve un nuevo objeto tipo combotableitem con la lista de tipos"""
        return ButtonTableItem(self.table2,self.dcasos)

        
    def __modificaciontablavar(self,x,y):
        """Funcion a que conecta con la introduccion de nuevos datos en la tabla de variables"""
        if y==1 and (self.table2.text(x,y).latin1()==self.__idu.getVar(x).tipo):
            return # Se ha solicitado un cambio de tipo al propio tipo
        self.__portero.guardarestado()
        if x>=self.__idu.numVar(): #Es una nueva variable
            #Variables intermedias (creadas con los valores por defecto)
            for valorintermedio in range (self.__idu.numVar(),x):  
                self.__insertarvariable()
                self.__showtablavar(valorintermedio)
            self.__insertarvariable()
        #Variable modificada
        variable = self.__idu.getVar(x)
        if y==1:   # El usuario quiere cambiar el tipo
            textoencuestion = self.table2.text(x,y).latin1() 
            #preguntar el tipo de conversion
            metododeseado = self.__preguntarconversion(variable, textoencuestion)
            if metododeseado:
                pareja = self.__agenteconversion(variable, textoencuestion, metododeseado) #Pareja variable-list
                self.__idu.setVar(x,pareja[0],pareja[1])
                self.__showtabladat()  
        else: #Restos de campos (Texto)
            from Driza.excepciones import VariableExisteException
            try:
                self.__idu.modVar(variable,y,str(self.table2.text(x,y).latin1()))
            except VariableExisteException:
                rc = QMessageBox.warning(self,'Atención', u'El nombre de variable ya existe')

        self.__showtablavar(x)  
        #En todos los casos, actualizamos el titulo de la tabla de datos
        self.__showtitulodat()

    def __actualizarreginterfazdatos(self,x,y,valor):
        """actualiza un dato en interfazdatos,recogiendo la excepcion en caso de error """
        try:
            self.__idu[x][y] = valor
        except ValueError:
            rc = QMessageBox.warning(self,'Atención', u'El dato no es válido')
    
    def __insertarregistro(self):
        """Inserta un registro genérico, y ademas comprueba que no nos estamos acercando al final de la tabla"""
        self.__idu.appReg()
        if self.__nreg>(self.table1.numRows()-self.__idu.numReg()):
            self.table1.setNumRows(self.table1.numRows()+1)

    def __insertarvariable(self):
        """Inserta una variable genérica, y ademas comprueba que no nos acercamos a ninguno de los limites de las tablas"""
        self.__idu.appVar()
        if self.__nvar>(self.table2.numRows()-self.__idu.numVar()):
            self.table2.setNumRows(self.table2.numRows()+1)
            self.table1.setNumCols(self.table1.numCols()+1)

    def __modificaciontablareg(self,x,y):
        """Actualiza los datos del objeto dato a partir de un cambio en la tabla de datos"""
        if (x < self.__idu.numReg())\
                and (y < self.__idu.numVar())\
                and (self.table1.text(x,y).latin1() == str(self.__idu[x][y])):
            return #No hay cambio efectivo #FIXME: No detecta reales
        self.__portero.guardarestado()
        valor=self.table1.text(x,y).latin1()
        if y>=self.__idu.numVar():
            #Estamos trabajando sobre una variable inexistente
            for i in range(self.__idu.numVar(),y+1):
                self.__insertarvariable()
                self.__showColumnaTablaReg(i)
            self.__showtablavar() #actualizamos la tabla de variables
        if x>=self.__idu.numReg():
            #no existen registros intermedios
            for i in range (self.__idu.numReg(),x):  
                self.__insertarregistro()
                self.__showtabladat(i)  
            self.__idu.appReg() #El último se separa porque tenemos que verificar si el usuario ha escrito correctamente el dato
        self.__actualizarreginterfazdatos(x,y,valor)
        self.__showtabladat(x)
        #Comprobacion de que sobra el numero correcto de celdas
        if self.__nreg>(self.table1.numRows()-self.__idu.numReg()):
            self.table1.setNumRows(self.__idu.numReg()+self.__nreg)
        self.parent().parent().parent().showundoredo() #FIXME... que son todos estos parent?

    
    def removeselection(self,borrardatos=False):
        """Esta funcion borra la seleccion de la tabla. El parametro opcional determina si se ha deborrar tambien los datos"""
        tablaactual=self.currentPageIndex()
        if tablaactual==0:
            tabla=self.table1
        else:
            tabla=self.table2
        if borrardatos:
            lista=[]
            for seleccion in range(tabla.numSelections()):
                for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
                    lista.append(fila)
            lista.sort()
            lista.reverse()
            for fila in lista:
                if tablaactual == 0:
                    del self.__idu[fila]
                else:
                    self.__idu.delVar(fila)

        for seleccion in range(tabla.numSelections()):
            tabla.removeSelection(seleccion)

    def __preguntarconversion(self,variable,objetivo):
        """Pregunta cual de los metodos disponibles para la conversion desea escoger el usuario"""
        from Driza.datos.agrupadores import Agrupador
        from Driza.datos.agrupadores import AgrupadorReal
        from Driza.datos.agrupadores import AgrupadorLogico
        from Driza.datos.agrupadores import AgrupadorEntero
        from Driza.datos.agrupadores import AgrupadorFactor
        from Driza.datos.agrupadores import AgrupadorOrdinal
        #Leer el diccionario de variable con la key objetivo
        tipoinicial="Agrupador"+objetivo
        lista=[]
        for tipo in variable.diccionarioconversion.iterkeys():
            if issubclass(eval(tipoinicial),eval(tipo)):
                lista+=variable.diccionarioconversion[tipo]
        #Elaborar una lista y preguntar al usuario
        qlista=QStringList()
        for elemento in lista:
            qlista.append(elemento)
        cadena=QInputDialog.getItem("Elige!",u"Elige una función de conversion",qlista,0,False,self,"Dialogo")
        #devolver el nombre de la funcion
        if cadena[1]:
            return cadena[0].latin1()
        else:
            return ""

    def listaseleccion(self,tabla=0):
        """Devuelve una lista con los registros que han sido seleccionados"""
        tabla=self.table1
        listasalida=[]
        if tabla.numSelections():
            toprow=tabla.numRows()
            bottomrow=0
            leftcol=tabla.numCols()
            rightcol=0
            for seleccion in range(tabla.numSelections()):
                misel = tabla.selection(seleccion)
                if misel.topRow()< toprow: toprow = misel.topRow()
                if misel.bottomRow()> bottomrow: bottomrow = misel.bottomRow()
                if misel.leftCol()< leftcol: leftcol = misel.leftCol()
                if misel.rightCol()> rightcol: rightcol = misel.rightCol()
                log.debug("Limites totales de seleccion:("+str(toprow)+","+str(leftcol)+")")
                log.debug("Limites totales de seleccion:("+str(bottomrow)+","+str(rightcol)+")")
            if bottomrow >= self.__idu.numReg(): bottomrow = self.__idu.numReg()-1
            if rightcol >= self.__idu.numVar(): rightcol = self.__idu.numVar()-1
            for x in range(toprow,bottomrow+1):
                nuevafila=[]
                for y in range(leftcol,rightcol+1):
                    if self.__idu[x][y].valido() and tabla.isSelected(x,y):
                        nuevafila.append(repr(self.__idu[x][y]))
                    else:
                        nuevafila.append(None)
                log.debug("Fila añadida a la seleccion de copiado: "+str(nuevafila))
                listasalida.append(nuevafila)
        return listasalida

    def insertarcaso(self):
        assert(self.currentPageIndex() == 0) #Estamos en la tabla de registros
        tabla = self.table1
        pos=tabla.currentRow()
        self.__idu.insReg(pos)
        self.myUpdate() #TODO: Optimizar redibujado

    def verificarSeleccionRegistros(self):
        """Verifica que la seleccion solamente consta de registros."""
        tabla = self.table1
        for indiceseleccion in range(tabla.numSelections()):
            seleccion=tabla.selection(indiceseleccion)
            if seleccion.leftCol()!=0 or seleccion.rightCol() < self.__idu.numVar()-1:
                log.debug("Seleccionincorrecta:"+str(seleccion.leftCol())+","+str(seleccion.rightCol()))
                from Driza.excepciones import SeleccionIncorrectaException
                raise SeleccionIncorrectaException

    def showtabcasos(self):
        """Muestra el tab de casos"""
        self.showPage(self.tab)

    def showtabvariables(self):
        """Muestra el tab de variables"""
        self.showPage(self.tab_2)
