#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from Driza.iuqt3.ui.vprincipal import vprincipal
from Driza.iuqt3.dconfig import DConfig
from Driza.iuqt3.dbuscar import DBuscar
from Driza.iuqt3.dfiltrado import DFiltrado
from Driza.iuqt3.dayuda import DAyuda
from grid import Grid
from Driza import funciones
from Driza import operaciones
from Driza import listas
from Driza.datos import datos
from Driza.datos import interfaces
import string
import logging
log = logging.getLogger("Driza.iuqt3.vprincipal.vprincipal")

class VPrincipal(vprincipal):
    """Ventana principal, hereda de vprincipal, que esta generado por el designer. Esta clase es padre de la mayoria de los dialogos, y es la que permite introducir los datos a el usuario"""

    def __init__(self,parent,config,portero,idu,gestorproyectos,gestortemas): 
        """Inicializador de la ventana principal"""
        vprincipal.__init__(self,None)
        #VARIABLES PUBLICAS
        self.parent = parent
        self.__config = config
        self.__gestortemas = gestortemas
        self.__idu = idu
        #Bloque de inicialización de dialogos
        self.__dbuscar = DBuscar(self,"busqueda")
        self.__dconfig = DConfig(config,self)
        self.__dfiltro = DFiltrado(self,self.__idu)
        self.__dayuda = DAyuda(self)
        self.__portero = portero
        self.__gestorproyectos = gestorproyectos
        self.grid = Grid(self,self.__idu,self.__portero)
        self.widgetStack1.addWidget(self.grid)
        self.widgetStack1.raiseWidget(self.grid)

        # Funciones de inicializacion de componentes
        self.__inittoolbar()
        self.showundoredo()

        self.setIcon(self.__gestortemas.iconoprograma())
        self.__atajos()

    #FUNCIONES PUBLICAS
    
    def abrirfichero(self,parent = None,filename = None):
        """ Lanza el diálogo de apertura de fichero """
        from Driza.excepciones import FicheroNoExisteException,FicheroErroneoException,FicheroTipoDesconocidoException
        if not parent: parent = self
        if not self.__preguntarproyectomodificado(): return
        if not filename:
            filter = ""
            for fmt in listas.extensiones_fichero():
                filter = filter + "%s files (*.%s);;" % (fmt, string.lower(fmt))
            fn = QFileDialog.getOpenFileName(QString.null,filter,self,None,"Dialogo abrir fichero","")
            filename = str(fn)
        if filename:
            try:
                self.__gestorproyectos.cargar(filename)
            except FicheroErroneoException:
                QMessageBox.warning(parent,u'Atención', 'El fichero no ha podido ser leido')
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(parent,u'Atención', 'El fichero no ha podido ser leido')
            except AttributeError:
                QErrorMessage(parent,"Error").message(u"Parece que ha intentado cargar un fichero de una versión anterior. Lo siento")
                log.exception("excepcion capturada")
            except:
                QErrorMessage(parent,"Error").message(u"Error desconocido")
            else:
                self.myUpdate()
                self.grid.myUpdate()
                return True
        return False

    def myUpdate(self):
        """Acciones de actualizacion"""
        if self.__gestorproyectos.fichero:
            nfichero= self.__gestorproyectos.fichero
        else:
            nfichero= "Nuevo fichero"
        self.setCaption("Driza " + self.parent.version() + " - " + nfichero)

    def showEvent(self,ev):
        self.myUpdate()
        vprincipal.showEvent(self,ev)

    def closeEvent( self, ev ):
        """Acciones que se realizaran cuando el usuario cierre la ventana. Guarda el estado de la configuración, incluyendo los ficheros recientemente abiertos"""
        #from Driza.config import InterfazConfig
        if not self.__preguntarproyectomodificado(): return
        QWidget.closeEvent( self, ev )
        if not self.parent.vsalida.isVisible():
            self.__config.save()
            qApp.exit(0)
        else:
            self.hide()


    
    def showundoredo(self):
        """Determina que botones estan activados y cuales no del submenu de edición"""
        if self.__portero.puedoundo():
            self.editUndoAction.setEnabled(True)
        else:
            self.editUndoAction.setEnabled(False)

        if self.__portero.puedoredo():
            self.editRedoAction.setEnabled(True)
        else:
            self.editRedoAction.setEnabled(False)

    #FUNCIONES PRIVADAS

    def __importar(self):
        """Importa los datos de un fichero"""
        if not self.__preguntarproyectomodificado(): return
        self.parent.dimportartexto.show()

    def __preguntarproyectomodificado(self):
        """Pregunta en caso de que haya sido modificado el proyecto si desea ser guardado"""
        if not self.__idu.original():
            rc = QMessageBox.information(self,'Atencion:', 'El proyecto actual ha sido modificado, desea guardarlo?', 'Guardarlo','No guardarlo','Volver',0,1)
            if rc==0:
                self.__guardar()
            elif rc==2:
                return False
        return True


    def __dacerca_de(self):
        """Mensaje acerca de"""
        separador = "\n----------------------------------\n"
        betatesters = u"Carlos Mestre Gonzalez\n Luis de Bethencourt Guimerá"
        iconos = "Iconos del Tango Desktop Project: http://tango.freedesktop.org/"
        ristra = u"Driza %s es una interfaz estadística\n (C) Néstor Arocha Rodríguez - Tutorizado por Inmaculada luengo Merino\n Distribuido bajo licencia GPL" +\
                separador + "Betatesters:\n" + betatesters + separador +\
                u" Mantenedor del paquete .deb:\n Luis de Bethencourt Guimerá"+ separador + iconos
        QMessageBox.about(self,"Acerca de Driza", ristra % (self.parent.version()))
    
    def __undo(self):
        """Deshace el último cambio"""
        self.__portero.undo()
        self.grid.myUpdate()
        self.showundoredo()
    
    def __redo(self):
        """Rehace el último cambio"""
        self.__portero.redo()
        self.grid.myUpdate()
        self.showundoredo()

    def __inittoolbar(self):
        """Inicializa la toolbar"""
        self.boton1=QToolButton(self.Toolbar)
        self.boton1.setTextLabel("ABRIR")
        iconos=QIconSet()
        iconos.setIconSize(QIconSet.Small,QSize(100,100))
        iconos.setPixmap(self.__gestortemas.iconoabrir(),QIconSet.Small)
#        iconos.setPixmap("images/flechaabajo.png",QIconSet.Automatic,QIconSet.Normal,QIconSet.Off)
        self.boton1.setOn(True)
        self.boton1.setIconSet(iconos)
        self.boton2=QToolButton(self.Toolbar)
        self.boton2.setTextLabel("GUARDAR")
        iconos=QIconSet()
        iconos.setIconSize(QIconSet.Small,QSize(100,100))
        iconos.setPixmap(self.__gestortemas.iconoguardar(),QIconSet.Small)
        self.boton2.setOn(True)
        self.boton2.setIconSet(iconos)
        self.boton3=QToolButton(self.Toolbar)
        self.boton3.setTextLabel("Ventana Salida")
        iconos=QIconSet()
        iconos.setIconSize(QIconSet.Small,QSize(100,100))
        iconos.setPixmap(self.__gestortemas.icononuevaventana(),QIconSet.Small)
        self.boton3.setOn(True)
        self.boton3.setIconSet(iconos)
        self.boton4=QToolButton(self.Toolbar)
        self.boton4.setTextLabel("Mostrar etiquetas")
        iconos=QIconSet()
        iconos.setIconSize(QIconSet.Small,QSize(100,100))
        iconos.setPixmap(self.__gestortemas.iconoetiquetas(),QIconSet.Small)
        self.boton4.setOn(True)
        self.boton4.setIconSet(iconos)

    def conexiones(self):
        """Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
        self.connect(self.archivoSalirAction,SIGNAL("activated()"),self.__salirprograma)
        self.connect(self.fileNewAction,SIGNAL("activated()"),self.__nuevo)
        self.connect(self.fileOpenAction,SIGNAL("activated()"),self.abrirfichero)
        self.connect(self.fileSaveAsAction,SIGNAL("activated()"),self.__guardarcomo)
        self.connect(self.fileSaveAction,SIGNAL("activated()"),self.__guardar)
        self.connect(self.archivoImportarAction,SIGNAL("activated()"),self.__importar)
        self.connect(self.archivoConfiguracinAction,SIGNAL("activated()"),self.__dconfig.show)
        self.connect(self.datosFiltrarAction,SIGNAL("activated()"),self.__dfiltro.show)
        self.connect(self.modificarCrear_nuevas_variablesAction,SIGNAL("activated()"),self.parent.dcrevar.show)
        self.connect(self.distribucionesDistribucion_NormalAction,SIGNAL("activated()"),self.__ddistribucionnormal)
        self.connect(self.editFindAction,SIGNAL("activated()"),self.__dbuscar.show)
        self.connect(self.editCopyAction,SIGNAL("activated()"),self.__copy)
        self.connect(self.editCutAction,SIGNAL("activated()"),self.__cut)
        self.connect(self.editPasteAction,SIGNAL("activated()"),self.__paste)
        self.connect(self.editUndoAction,SIGNAL("activated()"),self.__undo)
        self.connect(self.editRedoAction,SIGNAL("activated()"),self.__redo)
        self.connect(self.edicinBorrarAction,SIGNAL("activated()"),self.__borrar)
        self.connect(self.edicinInsertarAction,SIGNAL("activated()"),self.__insertar)
        self.connect(self.acerca_deAcerca_deAction,SIGNAL("activated()"),self.__dacerca_de)
        self.connect(self.boton1,SIGNAL("clicked()"),self.abrirfichero)
        self.connect(self.boton2,SIGNAL("clicked()"),self.__guardar)
        self.connect(self.boton3,SIGNAL("clicked()"),self.parent.vsalida.show)
        self.connect(self.boton4,SIGNAL("clicked()"),self.__toggleetiquetas)
        self.connect(self.ayudaAyudaAction,SIGNAL("activated()"),self.__mostrardayuda)
        self.__conexionesanalizar()


    def __conexionesanalizar(self):
        """Las conexiones del submenu "analizar" """
        self.connect(self.analizarMediaGrficosHistogramaAction,SIGNAL("activated()"),self.__dgraficoshistograma)
        self.connect(self.analizarMediaGrficosDiagrama_de_dispersinAction,SIGNAL("activated()"),self.__dgraficosdispersion)
        self.connect(self.analizarMediaDescriptivosAction,SIGNAL("activated()"),self.__dcalculodescriptivo)
        self.connect(self.analizarBondad_de_ajusteJi_cuadradoAction,SIGNAL("activated()"),self.__dcalculojicuadrado)
        self.connect(self.AnalizarKs,SIGNAL("activated()"),self.__dcalculoks)
        self.connect(self.analizarRegresin_linealRegresin_lineal_simpleAction,SIGNAL("activated()"),self.__dcalculoregresion)
        self.connect(self._2Comparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocompmedsim)
        self.connect(self._2Comparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompmeddob)
        self.connect(self.analizarPruebas_de_hiptesisProporcionesComparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocomppropsim)
        self.connect(self.analizarPruebas_de_hiptesisVarianzaComparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompvardob)
        self.connect(self.analizarPruebas_de_hiptesisVarianzanew_itemAction,SIGNAL("activated()"),self.__dcalculocompvarmul)
        self.connect(self.analizarMediaTabla_de_frecuenciasAction,SIGNAL("activated()"),self.__dtablafrecuencia)
        self.connect(self.analizarMediaGrficosDiagrama_de_barrasAction,SIGNAL("activated()"),self.__ddiagramabarras)
        self.connect(self._2ANOVAAction,SIGNAL("activated()"),self.__danova)

    def __atajos(self):
        """Establece los atajos"""
        self.ayudaAyudaAction.setAccel("F1")
        self.acciontab1=QAction(self,"tabcasos")
        self.connect(self.acciontab1,SIGNAL("activated()"),self.grid.showtabcasos)
        self.acciontab1.setAccel("F2")
        self.acciontab2=QAction(self,"tabvars")
        self.connect(self.acciontab2,SIGNAL("activated()"),self.grid.showtabvariables)
        self.acciontab2.setAccel("F3")
        self.analizarMediaDescriptivosAction.setAccel("F4")
        self.boton3.setAccel("F5") #Ventana salida
        self.archivoConfiguracinAction.setAccel("F10")
        self.acerca_deAcerca_deAction.setAccel("F12")

    def __copy(self):
        """Funcion que copia y borra la seleccion"""
        try:
            self.__copiar()
        except AssertionError:
            QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        except:
            QErrorMessage(parent,"Error").message(u"Error desconocido")
        else:
            self.grid.removeselection() 

   #http://www.google.com/codesearch?hl=en&q=+%22qtable%22+%22cut%22+%22paste%22+show:z9otKZeV6U8:R6dK3Cx-dYg:Gh37-3Ie27E&sa=N&cd=48&ct=rc&cs_p=http://mde.abo.fi/confluence/download/attachments/1011/coral-0.9.1.tar.gz&cs_f=coral-0.9.1/coral/modeler/property/propertyeditor.py#a0 
    def __cut(self):
        """Copia, borra la seleccion y su contenido"""
        from Driza.excepciones import SeleccionIncorrectaException
        try:
            self.grid.verificarSeleccionRegistros()
            self.__portero.guardarestado()
            self.__copiar()
        except SeleccionIncorrectaException:
            QMessageBox.warning(self,u'Atención', 'Solo se permite cortar registros')
        except AssertionError:
            QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        except:
            QErrorMessage(self,"Error").message(u"Error desconocido")
            log.exception("excepcion capturada en el cortado")
        else:
            self.grid.removeselection(borrardatos=True)
            self.grid.myUpdate()


    def __paste(self):
        """Funcion que envuelve a __pegar recogiendo sus excepciones"""
        try:
            self.__pegar()
        except:
            QErrorMessage(self,"Error").message(u"Error desconocido")
            log.exception("excepcion capturada en el pegado")

    def __pegar(self):
        """Pega lo que haya sido copiado"""
        cb = QApplication.clipboard()
        registro = cb.text().latin1()
        lista = eval(registro)
        posrow = self.grid.table1.currentRow()
        poscol = self.grid.table1.currentColumn()
        if len(lista[0])==self.__idu.numVar():
            poscol = 0 #Copia de registro completo
        else:
            #No permitimos pegar si el pegado implica variables nuevas
            assert(poscol+len(lista[0])<=self.__idu.numVar())
        self.__portero.guardarestado() #Guardamos el estado

        #Creamos nuevos registros hasta la posicion del cursor
        if posrow > self.__idu.numReg():
            for x in range(self.__idu.numReg(),posrow):
                self.__idu.appReg()
        #Hacemos el pegado efectivo
        i=posrow
        for registro in lista:
            if i >= self.__idu.numReg(): self.__idu.appReg()
            j=poscol
            for campo in registro:
                if campo!=None:
                    self.__idu[i][j]=campo
                j += 1
            i += 1
        self.grid.myUpdate()

    def __salirprograma(self):
        """Funcion de salida del programa. Guarda la configuracion"""
        from Driza.config import InterfazConfig
        if not self.__preguntarproyectomodificado(): return
        self.__config.save()
        qApp.exit(0)


    def __nuevo(self):
        """Accion realizada cuando el usuario clickea en Nuevo"""
        if not self.__preguntarproyectomodificado(): return
        self.__idu.borrartodo()
        self.__gestorproyectos.fichero = None
        self.myUpdate()
        self.grid.myUpdate()


    
    def __guardar(self):
        """Funcion de guardado del fichero. Si el fichero no es conocido se llama a guardarcomo"""
        if self.__gestorproyectos.fichero:
            self.__gestorproyectos.guardar(sobreescribir=True)
            self.__idu.setFlagoriginal()
        else:
            self.__guardarcomo()

    def __guardarcomo(self):
        """Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
        filter=""
        for fmt in listas.extensiones_fichero():
            filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
        fn = QFileDialog.getSaveFileName(QString.null,filter,self)
        filename = str(fn)
        if filename:
            from Driza.excepciones import FicheroExisteException,FicheroTipoDesconocidoException
            try:
                self.__gestorproyectos.guardar(filename)
            except FicheroExisteException,fichero:
                rc = QMessageBox.information(self,'Atencion:', 'El fichero'+fichero.fichero+' ya existe' , 'Sobreescribir','Otro nombre','Cancelar',0,1)
                if rc==0:
                    self.__gestorproyectos.guardar(filename,True)
                    self.__idu.setFlagoriginal()
                    self.myUpdate()
                elif rc==1:
                    self.__guardarcomo()
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(self,u'Atención', u'La extensión del fichero es incorrecta.\nPruebe con otra extensión')
                self.__gestorproyectos.fichero=None
            else:
                self.__idu.setFlagoriginal()
                self.myUpdate()

                    
    def __copiar(self):
        """Esta funcion pega el contenido del clipboard en la tabla actual. Es llamada por copy"""
        tablaactual=self.grid.currentPageIndex()
        assert(tablaactual == 0) #TODO Cambiar por una excepcion más concreta
        lista=self.grid.listaseleccion()
        cadena=repr(lista)
        cb = QApplication.clipboard()
        cb.setData(QTextDrag(cadena))
            

    def __dcalculodescriptivo(self):
        """Muestra el dialogo de calculo descriptivo"""
        self.parent.doperaciones.mostrar("Descriptivo")

    def __dtablafrecuencia(self):
        """Muestra"""
        self.parent.doperaciones.mostrar(u"Tabla de frecuencias")

    def __dcalculojicuadrado(self):
        """Muestra el dialogo ji cuadrado"""
        self.parent.doperaciones.mostrar("Ji Cuadrado")

    def __ddiagramabarras(self):
        """Muestra"""
        self.parent.doperaciones.mostrar(u"Diagrama de barras")

    def __dcalculoks(self):
        """Muestra el dialogo ks"""
        self.parent.doperaciones.mostrar("KS")

    def __dcalculoregresion(self):
        """Muestra el dialogo de regresion"""
        self.parent.doperaciones.mostrar(u"Regresión")

    def __dgraficoshistograma(self):
        """Muestra el la seccion de creación de histogramas"""
        self.parent.doperaciones.mostrar("Histograma")

    def __dgraficosdispersion(self):
        """Muestra el la seccion de creación de histogramas"""
        self.parent.doperaciones.mostrar(u"Diagrama de dispersión")

    def __ddistribucionnormal(self):
        """Muestra la distribucion normal"""
        self.parent.doperaciones.mostrar(u"Distribución Normal")

    def __dcalculocompmedsim(self):
        self.parent.doperaciones.mostrar(u"Comparación de medias simple")
        
    def __dcalculocompmeddob(self):
        self.parent.doperaciones.mostrar(u"Media de dos muestras independientes")

    def __dcalculocomppropsim(self):
        self.parent.doperaciones.mostrar(u"Comparación de proporciones simple")
        
    def __dcalculocomppropdob(self):
        self.parent.doperaciones.mostrar(u"Comparación de proporciones entre dos variables")

    def __dcalculocompvardob(self):
        self.parent.doperaciones.mostrar(u"Comparación de Varianzas entre dos variables")

    def __dcalculocompvarmul(self):
        self.parent.doperaciones.mostrar(u"Comparación de Varianzas entre múltiples variables")

    def __danova(self):
        self.parent.doperaciones.mostrar(u"Anova Simple")

    def __mostrardayuda(self):
        self.__dayuda.mostrar("vprincipal")

    def __borrar(self):
        self.grid.removeselection(borrardatos=True)
        self.grid.myUpdate()

    def __insertar(self):
        try:
            self.grid.insertarcaso()
        except:
            QMessageBox.warning(self,u'Atención', u'Error en la inserción')
            log.exception("excepcion capturada al insertar")

    def __toggleetiquetas(self):
        log.debug("Cambiado el modo etiqueta")
        self.grid.modoetiqueta = not self.grid.modoetiqueta
        self.grid.myUpdate()

