#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007   Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Clases de operaciones"""

import logging
log = logging.getLogger("Driza.operaciones")


class GestorOperaciones(dict):
    """
    Gestiona las operaciones que están activas en la ejecución 
    del programa
    """
    def __init__(self, dato, config):
        dict.__init__(self)
        self.__dato = dato
        self.__config = config
        self.__initintegrado() 

    def __initintegrado(self):
        """Esta funcion carga en el diccionario las operaciones"""
        modules=self.__listamodulos()
        for operacion in modules:
            try:
                tipo = operacion.tipo
                widget = operacion.widget
                definicionresultado = self.__cargardefiniciondedescripcion(operacion.nombre,operacion.definicionresultado)
                if tipo == "Calculo":
                    mioperacion = OperacionCalculo(operacion.nombre, \
                            self.__dato, self.__config, operacion.descripcion, widget, \
                            definicionresultado,operacion.funcionprincipal, \
                            operacion.funcionchequeocondiciones, \
                            operacion.funcionchequeoentradausuario)    
                elif tipo == "Distribucion":
                    mioperacion = OperacionDistribucion(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Graficos":
                    mioperacion = OperacionGraficos(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Tabla":
                    mioperacion = OperacionTabla(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                elif tipo == "Multiple":
                    mioperacion = OperacionMultiple(operacion.nombre, \
                            self.__dato, self.__config, widget, operacion.funcion, \
                            operacion.interfazresultado, \
                            operacion.initresultado,operacion.comprobarentrada)    
                self.__setitem__(operacion.nombre, mioperacion)

            except: #TODO: Poner el tipo de excepcion
                log.exception(u"Excepcion cargando el módulo:"+operacion.nombre)

    def __listamodulos(self):
        """Devuelve una lista con los modulos ya cargados"""
        #Sacado de:
        # http://www.diveintopython.org/functional_programming/all_together.html
        import Driza.carga.operaciones
        ruta = Driza.carga.operaciones.__path__[0]
        import os
        import re
        ficheros = os.listdir(ruta)
        del ficheros[ficheros.index("__init__.py")]
        test = re.compile(".*\.py$", re.IGNORECASE)
        ficheros = filter(test.search, ficheros)   
        filenametomodulename = lambda f: os.path.splitext(f)[0]
        modulenames = map(filenametomodulename, ficheros)
        mifuncion = lambda f: "Driza.carga.operaciones."+f
        modulenames = map(mifuncion, modulenames)
        mifuncion2 = lambda f: __import__(f, None, None, ["Driza.carga.operaciones"])
        modules=[]
        for nombremodulo in modulenames:
            try:
                modules.append(mifuncion2(nombremodulo))
            except:
                log.exception(u"Excepcion cargando el modulo del sistema: "+nombremodulo)
        return modules


    def __cargardefiniciondedescripcion(self,nombre,listadiccionariodefinicion):
        """Carga la definicion de los resultados a partir del diccionario suministrado por el modulo de la operacion"""
        from Driza.salida.base import FormatoResultado
        fresultado=FormatoResultado(nombre,self.__config)
        for diccionariodefinicion in listadiccionariodefinicion:
            elemento = None
            if diccionariodefinicion["tipo"]=="Texto":
                from Driza.salida import DefinicionElementoResultadoParrafo
                elemento=DefinicionElementoResultadoParrafo(diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Tabla":
                from Driza.salida.componenteresultado import DefinicionElementoResultadoTabla
                elemento=DefinicionElementoResultadoTabla(diccionariodefinicion["nombre"],diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Fila":
                from Driza.salida import DefinicionElementoResultadoFila
                elemento=DefinicionElementoResultadoFila(diccionariodefinicion["nombre"],diccionariodefinicion) #Que coja el los campos del diccionario
            elif diccionariodefinicion["tipo"]=="Imagen":
                from Driza.salida import DefinicionElementoResultadoImagen
                elemento=DefinicionElementoResultadoFila(diccionariodefinicion) #Que coja el los campos del diccionario
            fresultado[diccionariodefinicion["nombre"]]=elemento
        return fresultado

    def funcionchequeocondiciones(self,nombre):
        """Realiza el chequeo de condiciones llamando a la funcion suministrada por la operacion, pasando como parametros la interfazdedatos"""
        return self.__getitem__(nombre).funcionchequeocondiciones(self.__dato)



class Operacion:
    """Clase base de las operaciones. Cada subtipo reimplementa el procedimiento según lo necesita"""
    def __init__(self,nombre,interfazdatosr,config,descripcion,widget,listadefinicionresultado,funcionprincipal,funcionchequeocondiciones,funcionchequeoentradausuario):
        self.dato=interfazdatosr
        self.nombre=nombre
        self.listadefinicionresultado=listadefinicionresultado
        self.funcion=funcionprincipal
        self.widget=widget
        self.funcionchequeocondiciones=funcionchequeocondiciones
        self.funcionchequeoentradausuario=funcionchequeoentradausuario
        from Driza.salida.textrender import FloatRender
        self._fr=FloatRender(config)

class OperacionCalculo(Operacion):
    """Los calculos que se realizan sobre un conjunto de variables por separado"""
    def procedimiento(self,seleccion,opciones):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        listaelementos=seleccion
        if not listaelementos:
            raise IndexError #TODO Cambiar por execpción mejor
        if not self.funcionchequeoentradausuario(opciones):
            from Driza.excepciones import OpcionesIncorrectaException
            raise OpcionesIncorrectaException
        lista=[]
        #Guardamos en lista cada pareja, enunciado-resultado
        for elemento in listaelementos:
            lista.append(self.funcion(self.dato,elemento,opciones))
       
        
        #Creamos el resultado de salida
        resultado=self.listadefinicionresultado.renderizar(lista)

        #Devolvemos el objeto
        return resultado

#TODO Rediseñar
class GestorMensajesCategorias(dict):
    """Este gestor guarda un mensaje asociado a cada categoria.
    Se utiliza los operadores de diccionario para [] acceso y modificación
    Devuelve un QLabel con el texto"""
    def __init__(self):
        dict.__init__(self)

    def __setitem__(self, indice, parametro):
        if isinstance(parametro,str) or isinstance(parametro,unicode):
            label=None
            #from PyQt4.QtGui import QLabel
            #label=QLabel(None)
            #label.setText(parametro)
            dict.__setitem__(self,indice,label)
      
