#!/usr/bin/python
# -*- coding: utf-8 -*-


#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino 

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by

#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Componentes de los resultados en ventana salida"""

class ComponenteSalida:
    """Cualquier elemento que figure en un resultado será hijo de esta clase"""
    def __init__(self):
        self.contenido = None
        self.titulo = ""

    def set(self, diccionario):
        """Abstracta: define como ha de recibir cada elemento la información del diccionario dict"""
        pass

    def html(self):
        """Abstracta: muestra el contenido del widget volcado a html"""
        return ""

    def exportarhtml(self, fichero):
        """Esta funcion es llamada cuando se desea exportar html a un fichero. Normalmente hace lo mismo que html()"""
        return self.html()

    def settitulo(self, titulo):
        """Establece el valor del titulo"""
        if not isinstance(titulo, str):
            raise TypeError
        self.titulo = titulo

class Tabla:
    """Clase base de las tablas. Almacena una tabla. Esta clase asegura que se cumple la restriccion del tamaño para todos los registros"""
    def __init__(self, autoencoger = False):
        self._contenido = list()
        self._ncols = None
        self._listatitulos = []
        self.autoencoger = autoencoger

    def setCabecera(self, args):
        self._listatitulos = args

    def append(self, args):
        """Introduce los argumentos en la tabla. Ha de coincidir en tamaño con ncols.Recibe ristras"""
        if not self._ncols:
            self._ncols = len(args)
        if len(args) != self._ncols:
            raise IndexError
        else:
            self._contenido.append(args)
    def _listavalidos(self):
        """Devuelve una lista con verdaderos y falsos que determinan si en esa posicion hay algún valor válido o no"""
        listafinal = []
        for x in range(len(self._listatitulos)):
            if self.autoencoger:
                vale = False
                for fila in self._contenido:
                    if fila[x] != None:
                        vale = True
            else:
                vale = True
            listafinal.append(vale)
        return listafinal



class TablaHorizontal(Tabla):
    """Tabla que muestra los resultados en disposicion horizontal"""
    def html(self):
        """Devuelve el contenido de la tabla en html"""
        cadena = ""
        listavalidos = self._listavalidos()
        if self._listatitulos:
            i = 0
            cadena += "<tr>\n"
            for titulo in self._listatitulos:
                if listavalidos[i]:
                    cadena += "<td>"+titulo+"</td>"
                i += 1
            cadena += "</tr>\n"

        for fila in self._contenido:
            cadena += "<tr>\n"
            i = 0
            for y in fila:
                if listavalidos[i]:
                    if isinstance(y, list) or isinstance(y, float)\
                            or isinstance(y, dict) or isinstance(y, int):
                        y = str(y)
                    cadena += "<td>" + y + "</td>\n"
                i += 1
            cadena += "</tr>\n"
        return cadena

class TablaVertical(Tabla):
    """Tabla que muestra los resultados en disposicion vertical"""
    def html(self):
        """Devuelve el contenido de la tabla en html"""
        listavalidos = self._listavalidos()
        listatmp = []
        if self._listatitulos:
            i = 0
            for titulo in self._listatitulos:
                if listavalidos[i]:
                    listatmp.append([titulo])
                i += 1
        for fila in self._contenido:
            i = 0
            j = 0
            for y in fila:
                if listavalidos[i]:
                    if isinstance(y, list) or isinstance(y, float)\
                            or isinstance(y, dict) or isinstance(y, int):
                        y = str(y)
                    listatmp[j].append(y)
                    j += 1 
                i += 1

        cadena = u""
        for fila in listatmp:
            cadena += "<tr>\n"
            for columna in fila:
                cadena += "<td>" + columna + "</td>\n"
            cadena += "</tr>\n"
        return cadena

class CSLista(ComponenteSalida, list):
    """Varios resultados agrupados"""
    def html(self):
        ristra = ""
        for x in self:
            ristra += x.html()
        return ristra

class CSTabla(ComponenteSalida):
    """Una tabla"""
    def __init__(self, autoencoger = False, disposicion = "Horizontal"):
        ComponenteSalida.__init__(self)
        if disposicion == "Horizontal":
            self.contenido = TablaHorizontal(autoencoger)
        elif disposicion == "Vertical":
            self.contenido = TablaVertical(autoencoger)
        else:
            self.contenido = TablaHorizontal(autoencoger)
            raise ValueError

    def html(self):
        ristra = ""
        ristra += "<h2>" + self.titulo + "</h2>"
        ristra += "<table border=\"1\">"
        ristra += self.contenido.html()
        ristra += "</table>"
        return ristra

    def append(self, listaargs):
        """Debe recibir ristras"""
        self.contenido.append(listaargs)

    def setCabecera(self, titulo):
        self.contenido.setCabecera(titulo)

class CSTexto(ComponenteSalida):
    """Es un texto cualquiera"""
    def __init__(self, **args):
        ComponenteSalida.__init__(self)
        self.contenido = ""
        self.size = None
        if args.has_key("size"):
            self.size = args["size"]

    def set(self, texto):
        if isinstance(texto, str):
            self.contenido = texto
        elif isinstance(texto, unicode):
            self.contenido = texto.encode('iso-8859-1', 'replace')
        else:
            self.contenido(str(texto))

    def html(self):
        if self.contenido:
            if self.size:
                ristra = "<font size=+" + str(self.size) + ">" + self.contenido + "</font><br>"
            else:
                ristra = self.contenido + "<br>"
            return unicode(ristra, 'iso-8859-1')
        else: 
            return u""

class CSImagen(ComponenteSalida):
    """Representa una imagen"""
    def __init__(self):
        ComponenteSalida.__init__(self)
        self.contenido = None #TODO
        self.ruta = None

    def set(self, ruta):
        self.ruta = ruta

    def exportarhtml(self, carpeta):
        """Copia el archivo a la carpeta donde estaran las imagenes, y devuelve el html con la ruta de la imagen actualizada
        La carpeta ya debe estar creada
        """
        import os, shutil
        ristra = ""
        if self.ruta:
            (dirName, fileName) = os.path.split(self.ruta)
            shutil.copyfile(self.ruta, carpeta + "/" + fileName)
            ristra += "<h2>" + self.titulo + "</h2>"
            ristra += "<img src=\"" + os.path.split(carpeta)[1] + "/" + fileName + "\">"
        return ristra

    def html(self):
        ristra = ""
        if self.ruta:
            ristra += "<h2>" + self.titulo + "</h2>"
            ristra += "<img src=\"" + self.ruta + "\">"
        return ristra


